@@ -1,6 +1,6 @@
 # Java skeleton for Bison                           -*- autoconf -*-
 
-# Copyright (C) 2007-2015, 2018-2020 Free Software Foundation, Inc.
+# Copyright (C) 2007-2015, 2018-2021 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 m4_include(b4_skeletonsdir/[java.m4])
 
@@ -53,8 +53,9 @@ b4_use_push_for_pull_if([
 # allows them to be defined either in parse() when doing pull parsing,
 # or as class instance variable when doing push parsing.
 m4_define([b4_define_state],[[
-    /* Lookahead and lookahead in internal form.  */
+    /* Lookahead token kind.  */
     int yychar = YYEMPTY_;
+    /* Lookahead symbol kind.  */
     SymbolKind yytoken = null;
 
     /* State.  */
@@ -77,9 +78,9 @@ m4_define([b4_define_state],[[
 
 ]b4_output_begin([b4_parser_file_name])[
 ]b4_copyright([Skeleton implementation for Bison LALR(1) parsers in Java],
-              [2007-2015, 2018-2020])[
+              [2007-2015, 2018-2021])[
 ]b4_disclaimer[
-]b4_percent_define_ifdef([package], [package b4_percent_define_get([package]);[
+]b4_percent_define_ifdef([api.package], [package b4_percent_define_get([api.package]);[
 ]])[
 ]b4_user_pre_prologue[
 ]b4_user_post_prologue[
@@ -602,9 +603,9 @@ b4_dollar_popdef[]dnl
             push_token_consumed = false;]], [b4_parse_trace_if([[
             yycdebug ("Reading a token");]])[
             yychar = yylexer.yylex ();
-            yylval = yylexer.getLVal ();]b4_locations_if([
-            yylloc = new b4_location_type (yylexer.getStartPos (),
-                            yylexer.getEndPos ());])[
+            yylval = yylexer.getLVal();]b4_locations_if([[
+            yylloc = new ]b4_location_type[(yylexer.getStartPos(),
+                                          yylexer.getEndPos());]])[
 ]])[
           }
 
@@ -613,14 +614,14 @@ b4_dollar_popdef[]dnl
         yySymbolPrint("Next token is", yytoken,
                       yylval]b4_locations_if([, yylloc])[);]])[
 
-        if (yytoken == SymbolKind.]b4_symbol_prefix[YYerror)
+        if (yytoken == ]b4_symbol(1, kind)[)
           {
             // The scanner already issued an error message, process directly
             // to error recovery.  But do not keep the error token as
             // lookahead, it is too special and may lead us to an endless
             // loop in error recovery. */
-            yychar = Lexer.]b4_percent_define_get([api.token.prefix])[YYUNDEF;
-            yytoken = SymbolKind.]b4_symbol_prefix[YYUNDEF;]b4_locations_if([[
+            yychar = Lexer.]b4_symbol(2, id)[;
+            yytoken = ]b4_symbol(2, kind)[;]b4_locations_if([[
             yyerrloc = yylloc;]])[
             label = YYERRLAB1;
           }
@@ -628,8 +629,8 @@ b4_dollar_popdef[]dnl
           {
             /* If the proper action on seeing token YYTOKEN is to reduce or to
                detect an error, take that action.  */
-            yyn += yytoken.getCode ();
-            if (yyn < 0 || YYLAST_ < yyn || yycheck_[yyn] != yytoken.getCode ())
+            yyn += yytoken.getCode();
+            if (yyn < 0 || YYLAST_ < yyn || yycheck_[yyn] != yytoken.getCode())
               label = YYDEFAULT;
 
             /* <= 0 means reduce or error.  */
@@ -744,9 +745,9 @@ b4_dollar_popdef[]dnl
             yyn = yypact_[yystate];
             if (!yyPactValueIsDefault (yyn))
               {
-                yyn += SymbolKind.]b4_symbol(1, kind)[.getCode ();
+                yyn += ]b4_symbol(1, kind)[.getCode();
                 if (0 <= yyn && yyn <= YYLAST_
-                    && yycheck_[yyn] == SymbolKind.]b4_symbol(1, kind)[.getCode ())
+                    && yycheck_[yyn] == ]b4_symbol(1, kind)[.getCode())
                   {
                     yyn = yytable_[yyn];
                     if (0 < yyn)
@@ -779,7 +780,7 @@ b4_dollar_popdef[]dnl
         yystack.pop (2);]])[
 
         /* Shift the error token.  */]b4_parse_trace_if([[
-        yySymbolPrint("Shifting", SymbolKind.get (yystos_[yyn]),
+        yySymbolPrint("Shifting", SymbolKind.get(yystos_[yyn]),
                       yylval]b4_locations_if([, yyloc])[);]])[
 
         yystate = yyn;
@@ -933,17 +934,19 @@ b4_dollar_popdef[]dnl
           int yychecklim = YYLAST_ - yyn + 1;
           int yyxend = yychecklim < NTOKENS ? yychecklim : NTOKENS;
           for (int yyx = yyxbegin; yyx < yyxend; ++yyx)
-            if (yycheck_[yyx + yyn] == yyx && yyx != SymbolKind.]b4_symbol(1, kind)[.getCode ()
-                && !yyTableValueIsError (yytable_[yyx + yyn]))
+            if (yycheck_[yyx + yyn] == yyx && yyx != ]b4_symbol(1, kind)[.getCode()
+                && !yyTableValueIsError(yytable_[yyx + yyn]))
               {
                 if (yyarg == null)
                   yycount += 1;
                 else if (yycount == yyargn)
                   return 0; // FIXME: this is incorrect.
                 else
-                  yyarg[yycount++] = SymbolKind.get (yyx);
+                  yyarg[yycount++] = SymbolKind.get(yyx);
               }
         }
+      if (yyarg != null && yycount == yyoffset && yyoffset < yyargn)
+        yyarg[yycount] = null;
       return yycount - yyoffset;
     }
   }
@@ -979,12 +982,12 @@ b4_dollar_popdef[]dnl
          to an error action in a later state.
     */
     int yycount = 0;
-    if (yyctx.getToken () != null)
+    if (yyctx.getToken() != null)
       {
         if (yyarg != null)
-          yyarg[yycount] = yyctx.getToken ();
+          yyarg[yycount] = yyctx.getToken();
         yycount += 1;
-        yycount += yyctx.getExpectedTokens (yyarg, 1, yyargn);
+        yycount += yyctx.getExpectedTokens(yyarg, 1, yyargn);
       }
     return yycount;
   }
@@ -1069,27 +1072,28 @@ b4_dollar_popdef[]dnl
     /* The symbols being reduced.  */
     for (int yyi = 0; yyi < yynrhs; yyi++)
       yySymbolPrint("   $" + (yyi + 1) + " =",
-                    SymbolKind.get (yystos_[yystack.stateAt (yynrhs - (yyi + 1))]),
+                    SymbolKind.get(yystos_[yystack.stateAt (yynrhs - (yyi + 1))]),
                     ]b4_rhs_data(yynrhs, yyi + 1)b4_locations_if([,
                     b4_rhs_location(yynrhs, yyi + 1)])[);
   }]])[
 
   /* YYTRANSLATE_(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
      as returned by yylex, with out-of-bounds checking.  */
-  private static final SymbolKind yytranslate_ (int t)
+  private static final SymbolKind yytranslate_(int t)
 ]b4_api_token_raw_if(dnl
 [[  {
-    return SymbolKind.get (t);
+    return SymbolKind.get(t);
   }
 ]],
 [[  {
-    int user_token_number_max_ = ]b4_user_token_number_max[;
+    // Last valid token kind.
+    int code_max = ]b4_code_max[;
     if (t <= 0)
-      return SymbolKind.]b4_symbol_prefix[YYEOF;
-    else if (t <= user_token_number_max_)
-      return SymbolKind.get (yytranslate_table_[t]);
+      return ]b4_symbol(0, kind)[;
+    else if (t <= code_max)
+      return SymbolKind.get(yytranslate_table_[t]);
     else
-      return SymbolKind.]b4_symbol_prefix[YYUNDEF;
+      return ]b4_symbol(2, kind)[;
   }
   ]b4_integral_parser_table_define([translate_table], [b4_translate])[
 ]])[
