@@ -1,6 +1,6 @@
 /* File names and variables for bison,
 
-   Copyright (C) 1984, 1989, 2000-2002, 2006-2007, 2009-2015, 2018-2019
+   Copyright (C) 1984, 1989, 2000-2002, 2006-2007, 2009-2015, 2018-2021
    Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef FILES_H_
 # define FILES_H_
@@ -50,9 +50,15 @@ extern char *spec_xml_file;
 /* File name specified with --defines.  */
 extern char *spec_header_file;
 
+/* File name specified with --defines, adjusted for mapped prefixes. */
+extern char *spec_mapped_header_file;
+
 /* Directory prefix of output file names.  */
 extern char *dir_prefix;
 
+/* Directory prefix of output file name, adjusted for mapped prefixes. */
+extern char *mapped_dir_prefix;
+
 /* The file name as given on the command line.
    Not named "input_file" because Flex uses this name for an argument,
    and therefore GCC warns about a name clash. */
@@ -64,6 +70,9 @@ extern char *all_but_ext;
 /* Where our data files are installed.  */
 char const *pkgdatadir (void);
 
+/* Where the m4 program is installed.  */
+char const *m4path (void);
+
 void compute_output_file_names (void);
 void output_file_names_free (void);
 
@@ -82,4 +91,7 @@ FILE *xfopen (const char *name, char const *mode);
 void xfclose (FILE *ptr);
 FILE *xfdopen (int fd, char const *mode);
 
+char *map_file_name (char const *filename);
+void add_prefix_map (char const *oldprefix, char const *newprefix);
+
 #endif /* !FILES_H_ */
